@@ -66,6 +66,7 @@
 /* Stamp files to know crawling/indexing state */
 #define FIRST_INDEX_FILENAME          "first-index.txt"
 #define LAST_CRAWL_FILENAME           "last-crawl.txt"
+#define NEED_MTIME_CHECK_FILENAME     "no-need-mtime-check.txt"
 
 typedef enum {
 	TRACKER_DB_LOCATION_DATA_DIR,
@@ -162,6 +163,7 @@ static GStaticPrivate        interface_data_key = G_STATIC_PRIVATE_INIT;
 
 /* mutex used by singleton connection in libtracker-direct, not used by tracker-store */
 static GStaticMutex          global_mutex = G_STATIC_MUTEX_INIT;
+static TrackerDBInterface   *global_iface;
 
 static const gchar *
 location_to_directory (TrackerDBLocation location)
@@ -183,17 +185,12 @@ db_exec_no_reply (TrackerDBInterface *iface,
                   const gchar        *query,
                   ...)
 {
-	TrackerDBResultSet *result_set;
 	va_list                     args;
 
 	va_start (args, query);
-	result_set = tracker_db_interface_execute_vquery (iface, NULL, query, args);
+	tracker_db_interface_execute_vquery (iface, NULL, query, args);
 	va_end (args);
 
-	if (result_set) {
-		g_object_unref (result_set);
-	}
-
 	return TRUE;
 }
 
@@ -230,7 +227,6 @@ db_set_params (TrackerDBInterface *iface,
 		g_free (queries);
 	} else {
 		GError *error = NULL;
-		TrackerDBResultSet *result_set;
 
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA synchronous = OFF;");
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA count_changes = 0;");
@@ -238,14 +234,12 @@ db_set_params (TrackerDBInterface *iface,
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = \"UTF-8\"");
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA auto_vacuum = 0;");
 
-		result_set = tracker_db_interface_execute_query (iface, &error, "PRAGMA journal_mode = WAL;");
-		if (result_set == NULL) {
+		tracker_db_interface_execute_query (iface, &error, "PRAGMA journal_mode = WAL;");
+		if (error) {
 			/* Don't just silence the problem. This pragma must return 'WAL' */
 			g_message ("Can't set journal mode to WAL: '%s'",
-			           error ? error->message : "unknown error");
+			           error->message);
 			g_clear_error (&error);
-		} else {
-			g_object_unref (result_set);
 		}
 
 		if (page_size != TRACKER_DB_PAGE_SIZE_DONT_SET) {
@@ -410,6 +404,7 @@ db_manager_remove_all (gboolean rm_journal)
 	/* Remove stamp files */
 	tracker_db_manager_set_first_index_done (FALSE);
 	tracker_db_manager_set_last_crawl_done (FALSE);
+	tracker_db_manager_set_need_mtime_check (TRUE);
 
 	/* NOTE: We don't have to be initialized for this so we
 	 * calculate the absolute directories here.
@@ -431,8 +426,6 @@ db_manager_remove_all (gboolean rm_journal)
 	}
 
 	if (rm_journal) {
-
-
 		db_manager_remove_journal ();
 
 		/* If also the journal is gone, we can also remove db-version.txt, it
@@ -444,17 +437,6 @@ db_manager_remove_all (gboolean rm_journal)
 	db_remove_locale_file ();
 }
 
-void
-tracker_db_manager_remove_version_file (void)
-{
-	gchar *filename;
-
-	filename = g_build_filename (data_dir, TRACKER_DB_VERSION_FILE, NULL);
-	g_message ("  Removing db-version file:'%s'", filename);
-	g_unlink (filename);
-	g_free (filename);
-}
-
 static TrackerDBVersion
 db_get_version (void)
 {
@@ -494,8 +476,8 @@ db_get_version (void)
 	return version;
 }
 
-static void
-db_set_version (void)
+void
+tracker_db_manager_create_version_file (void)
 {
 	GError *error = NULL;
 	gchar  *filename;
@@ -516,6 +498,17 @@ db_set_version (void)
 	g_free (filename);
 }
 
+void
+tracker_db_manager_remove_version_file (void)
+{
+	gchar *filename;
+
+	filename = g_build_filename (data_dir, TRACKER_DB_VERSION_FILE, NULL);
+	g_message ("  Removing db-version file:'%s'", filename);
+	g_unlink (filename);
+	g_free (filename);
+}
+
 static void
 db_remove_locale_file (void)
 {
@@ -822,7 +815,8 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	}
 
 	if (need_reindex) {
-		db_set_version ();
+		tracker_db_manager_create_version_file ();
+		tracker_db_manager_set_need_mtime_check (TRUE);
 	}
 
 	g_message ("Checking database files exist");
@@ -1034,6 +1028,8 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	if (flags & TRACKER_DB_MANAGER_READONLY) {
 		resources_iface = tracker_db_manager_get_db_interfaces_ro (1,
 		                                                           TRACKER_DB_METADATA);
+		/* libtracker-direct does not use per-thread interfaces */
+		global_iface = resources_iface;
 	} else {
 		resources_iface = tracker_db_manager_get_db_interfaces (1,
 		                                                        TRACKER_DB_METADATA);
@@ -1050,7 +1046,9 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	s_cache_size = select_cache_size;
 	u_cache_size = update_cache_size;
 
-	g_static_private_set (&interface_data_key, resources_iface, (GDestroyNotify) g_object_unref);
+	if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
+		g_static_private_set (&interface_data_key, resources_iface, (GDestroyNotify) g_object_unref);
+	}
 
 	return TRUE;
 }
@@ -1084,6 +1082,12 @@ tracker_db_manager_shutdown (void)
 	sys_tmp_dir = NULL;
 	g_free (sql_dir);
 
+	if (global_iface) {
+		/* libtracker-direct */
+		g_object_unref (global_iface);
+		global_iface = NULL;
+	}
+
 	/* shutdown db interfaces in all threads */
 	g_static_private_free (&interface_data_key);
 
@@ -1501,6 +1505,12 @@ tracker_db_manager_get_db_interface (void)
 	TrackerDBInterface *interface;
 
 	g_return_val_if_fail (initialized != FALSE, NULL);
+
+	if (global_iface) {
+		/* libtracker-direct */
+		return global_iface;
+	}
+
 	interface = g_static_private_get (&interface_data_key);
 
 	/* Ensure the interface is there */
@@ -1696,6 +1706,96 @@ tracker_db_manager_set_last_crawl_done (gboolean done)
 	g_free (filename);
 }
 
+inline static gchar *
+get_need_mtime_check_filename (void)
+{
+	return g_build_filename (g_get_user_cache_dir (),
+	                         "tracker",
+	                         NEED_MTIME_CHECK_FILENAME,
+	                         NULL);
+}
+
+/**
+ * tracker_db_manager_get_need_mtime_check:
+ *
+ * Check if the miner-fs was cleanly shutdown or not.
+ *
+ * Returns: %TRUE if we need to check mtimes for directories against
+ * the database on the next start for the miner-fs, %FALSE otherwise.
+ *
+ * Since: 0.10
+ **/
+gboolean
+tracker_db_manager_get_need_mtime_check (void)
+{
+	gboolean exists;
+	gchar *filename;
+
+	filename = get_need_mtime_check_filename ();
+	exists = g_file_test (filename, G_FILE_TEST_EXISTS);
+	g_free (filename);
+
+	/* Existence of the file means we cleanly shutdown before and
+	 * don't need to do the mtime check again on this start.
+	 */
+	return !exists;
+}
+
+/**
+ * tracker_db_manager_set_need_mtime_check:
+ * @needed: a #gboolean
+ *
+ * If the next start of miner-fs should perform a full mtime check
+ * against each directory found and those in the database (for
+ * complete synchronisation), then @needed should be #TRUE, otherwise
+ * #FALSE.
+ *
+ * Creates a file in $HOME/.cache/tracker/ if an mtime check is not
+ * needed. The idea behind this is that a check is forced if the file
+ * is not cleaned up properly on shutdown (i.e. due to a crash or any
+ * other uncontrolled shutdown reason).
+ *
+ * Since: 0.10
+ **/
+void
+tracker_db_manager_set_need_mtime_check (gboolean needed)
+{
+	gboolean already_exists;
+	gchar *filename;
+
+	filename = get_need_mtime_check_filename ();
+	already_exists = g_file_test (filename, G_FILE_TEST_EXISTS);
+
+	/* !needed = add file
+	 *  needed = remove file
+	 */
+	if (!needed && !already_exists) {
+		GError *error = NULL;
+
+		/* Create stamp file if not already there */
+		if (!g_file_set_contents (filename, PACKAGE_VERSION, -1, &error)) {
+			g_warning ("  Could not create file:'%s' failed, %s",
+			           filename,
+			           error->message);
+			g_error_free (error);
+		} else {
+			g_message ("  Need mtime check file:'%s' created",
+			           filename);
+		}
+	} else if (needed && already_exists) {
+		/* Remove stamp file */
+		g_message ("  Removing need mtime check file:'%s'", filename);
+
+		if (g_remove (filename)) {
+			g_warning ("    Could not remove file:'%s', %s",
+			           filename,
+			           g_strerror (errno));
+		}
+	}
+
+	g_free (filename);
+}
+
 void
 tracker_db_manager_lock (void)
 {
